function [out,pixdim,dtype,dimension] = read_analyze(filename,RESHAPE_DATA,FLIP_DATA)
// READANALYZE  - Read MRI files in the Analyze format
//
//    usage: [data,pixdim,dtype] = readanalyze(filename);
//
//        filename - is the name of the file
//              (both a header (.hdr) and image (.img) file must exist)
//              (filename can include either the .hdr or .img
//              extensions or just the base filename.)
//
//            data - is a n-dimensional matrix of the data
//          pixdim - is the size of each voxel (mm)
//           dtype - datatype (ie 'uint8', 'int16', etc)
//

// Written by Colin Humphries
//   Jan, 2000
//   University of California, Irvine
//   colin@alumni.caltech.edu



// Info on Analyze format: 
//    http://www.mayo.edu/bir/analyze/AnalyzeFileInfo.html
//    Note: I checked this link and it no longer works. There should be
//    some info on the Analyze web site.

// Translated in Scilab and modified by Davide Poggiali, PhD student Padua IT, June 2014




// //////// User parameters //////////////////////////////////////////
if argn(2)<2 then
RESHAPE_DATA = 1; // When this flag is set to 1 the data is reshaped into
                  // an n-dimensional matrix. Otherwise readanalyze will
                  // just output a vector. 
end

if argn(2)<3 then
FLIP_DATA = 0;    // When this flag is set to 1 the outputed image is
                  // flipped in the first three dimensions.
end
                  stacksize('max')
// ////////////////////////////////////////////////////////////////////////////////////

// Note: FLIP_DATA has no effect when RESHAPE_DATA is 0.

// Get rid of possible file extensions
ind = mtlb_findstr(filename,'.img');
if ~isempty(ind)
  filename = filename(1:ind-1);
end

ind = mtlb_findstr(filename,'.hdr');
if ~isempty(ind)
  filename = filename(1:ind-1);
end

// Open Header
fid = mopen(filename+'.hdr','r');
if fid < 0
  error('Error opening header file');
end
mseek(40,fid,'set');
dimension =  mtlb_fread(fid,8,'int16');
byteswap = 0;

// The next part tries to figure out if we have a byte swapping problem by
// looking at the dimension bit in the header file. If it's bigger than 10
// (we probably aren't dealing with more than 10-dimensional data) then it
// tries byte swapping. If it still fails then it outputs an error.
if (dimension(1) > 10)
  byteswap = 'b';
  mclose(fid);
  fid = mopen(filename+'.hdr','r','b');
  mseek(40,fid,'set');
  dimension =  mtlb_fread(fid,8,'int16');
  if (dimension(1) > 10)
    byteswap = 'l'
    mclose(fid);
    fid = mopen(filename+'.hdr','r','l');
    mseek(40,fid,'set');
    dimension = mtlb_fread(fid,8,'int16');
    if (dimension(1) > 10)
      error('Error opening file. Dimension argument is not valid');
    end
  end
end

// Read information from header file
// datatype
mseek(40+30,fid,'set');
datatype = mtlb_fread(fid,1,'int16');

// pixel dimension
mseek(40+36,fid,'set');
pixdim = mtlb_fread(fid,8,'float');

// orient
mseek(40+108+104,fid,'set');
orient = mget(1,'ui',fid);
// Currently orient is not used.
//  orient:     slice orientation for this dataset. 
//       0      transverse unflipped 
//       1      coronal unflipped 
//       2      sagittal unflipped 
//       3      transverse flipped 
//       4      coronal flipped 
//       5      sagittal flipped 

// Close header file
mclose(fid);

// Open Data File
fid = mopen(filename+'.img','r',byteswap);

if fid < 0
  error('Error opening data file');
end

select datatype
  case 2
    dtype = 'uint8';
  case 4
    dtype = 'int16';
  case 8
    dtype = 'int32';
  case 16
    dtype = 'float';
  case 64
    dtype = 'double';
  case 132
    dtype = 'int16';
  else
    error('Unsupported datatype');
end

// Read binary data
out = mtlb_fread(fid,prod(dimension(2:$)),dtype);

// Close data file
mclose(fid);

if RESHAPE_DATA
  // Reshape data
  select dimension(1)
    case 4
      // 4-dimensional data
      if dimension(5) == 1
	out = matrix(out,dimension(2),dimension(3),dimension(4));
	if FLIP_DATA
	  out = flipdim(out,1);
	  out = flipdim(out,2);
	  out = flipdim(out,3);
	end
	// //////////////////////////////////////
      else
	out = matrix(out,dimension(2),dimension(3),dimension(4),dimension(5));
	if FLIP_DATA
	  out = flipdim(out,1);
	  out = flipdim(out,2);
	  out = flipdim(out,3);
	end
      end
    case 3
      // 3-dimensional data
      out = matrix(out,dimension(2),dimension(3),dimension(4));
      if FLIP_DATA
	out = flipdim(out,1);
	out = flipdim(out,2);
	out = flipdim(out,3);
      end
      // //////////////////////////////////////
    case 2
      // 2-dimensional data
      out = matrix(out,dimension(2),dimension(3));
    case 1
      // 1-d data
    else
      printf('Warning: data not reshaped\n');
  end
end

ii = find(pixdim);
pixdim = pixdim(ii);
  

save(filename+'.dat',out,pixdim,dtype,dimension);
  
endfunction
