function [I,pixdim,dtype,dimension]=loadimage(filename)
    // Loads the .img file or the corresponding .dat file if exists
    filename=strsubst(filename,".hdr","");
    filename=strsubst(filename,".img","");
    filename=strsubst(filename,".nii.gz","");
    filename=strsubst(filename,".nii","");
    filename=strsubst(filename,".dat","");
    if isfile(filename+".dat") 
        load(filename+".dat")
    elseif isfile(filename+".hdr")
        [I,pixdim,dtype,dimension]=read_analyze(filename);
    elseif isfile(filename+".nii")
        [I,pixdim,dtype,dimension]=read_nifti(filename+'.nii');
    elseif isfile(filename+".nii.gz")
        unix_w('gunzip '+filename+".nii.gz")
        unix_w('sleep 1')
        [I,pixdim,dtype,dimension]=loadimage(filename+'.nii');
    else
        disp("not found")
    end
    I=double(1*int32(I))
endfunction






function [I,pixdim,dtype,dimension] = read_analyze(filename,RESHAPE_DATA,FLIP_DATA)
    // READANALYZE  - Read MRI files in the Analyze format
    //
    //    usage: [data,pixdim,dtype] = readanalyze(filename);
    //
    //        filename - is the name of the file
    //              (both a header (.hdr) and image (.img) file must exist)
    //              (filename can include either the .hdr or .img
    //              extensions or just the base filename.)
    //
    //            data - is a n-dimensional matrix of the data
    //          pixdim - is the size of each voxel (mm)
    //           dtype - datatype (ie 'uint8', 'int16', etc)
    //

    // Written by Colin Humphries
    //   Jan, 2000
    //   University of California, Irvine
    //   colin@alumni.caltech.edu

    

    // Info on Analyze format: 
    //    http://www.mayo.edu/bir/analyze/AnalyzeFileInfo.html
    //    Note: I checked this link and it no longer works. There should be
    //    some info on the Analyze web site.

    // Modified and translated in Scilab by Davide Poggiali, PhD student Padua IT, June 2014
    // davide.poggiali@studenti.unipd.it



    // //////// User parameters //////////////////////////////////////////
    if argn(2)<2 then
        RESHAPE_DATA = 1; // When this flag is set to 1 the data is reshaped into
        // an n-dimensional matrix. Otherwise readanalyze will
        // just output a vector. 
    end

    if argn(2)<3 then
        FLIP_DATA = 0;    // When this flag is set to 1 the outputed image is
        // flipped in the first three dimensions.
    end


    stacksize('max')   // This line was added to allow big data loading
    // ////////////////////////////////////////////////////////////////////////////////////

    // Note: FLIP_DATA has no effect when RESHAPE_DATA is 0.

    // Get rid of possible file extensions
    filename=strsubst(filename,".hdr","");
    filename=strsubst(filename,".img","");

    // Open Header
    fid = mopen(filename+'.hdr','r');
    if fid < 0
        error('Error opening header file');
    end
    mseek(40,fid,'set');
    dimension =  mtlb_fread(fid,8,'int16');
    byteswap = 0;

    // The next part tries to figure out if we have a byte swapping problem by
    // looking at the dimension bit in the header file. If it's bigger than 10
    // (we probably aren't dealing with more than 10-dimensional data) then it
    // tries byte swapping. If it still fails then it outputs an error.
    if (dimension(1) > 10)
        byteswap = 'b';
        mclose(fid);
        fid = mopen(filename+'.hdr','r','b');
        mseek(40,fid,'set');
        dimension =  mtlb_fread(fid,8,'int16');
        if (dimension(1) > 10)
            byteswap = 'l'
            mclose(fid);
            fid = mopen(filename+'.hdr','r','l');
            mseek(40,fid,'set');
            dimension = mtlb_fread(fid,8,'int16');
            if (dimension(1) > 10)
                error('Error opening file. Dimension argument is not valid');
            end
        end
    end

    // Read information from header file
    // datatype
    mseek(40+30,fid,'set');
    datatype = mtlb_fread(fid,1,'int16');

    // pixel dimension
    mseek(40+36,fid,'set');
    pixdim = mtlb_fread(fid,8,'float');

    // orient
    mseek(40+108+104,fid,'set');
    orient = mget(1,'ui',fid);
    // Currently orient is not used.
    //  orient:     slice orientation for this dataset. 
    //       0      transverse unflipped 
    //       1      coronal unflipped 
    //       2      sagittal unflipped 
    //       3      transverse flipped 
    //       4      coronal flipped 
    //       5      sagittal flipped 

    // Close header file
    mclose(fid);

    // Open Data File
    fid = mopen(filename+'.img','r',byteswap);

    if fid < 0
        error('Error opening data file');
    end

    select datatype
    case 2
        dtype = 'uint8'; dtype2='us';
    case 4
        dtype = 'int16'; dtype2='s';
    case 8
        dtype = 'int32'; dtype2='i';
    case 16
        dtype = 'float'; dtype2='f';
    case 64
        dtype = 'double'; dtype2='d';
    case 132
        dtype = 'int16'; dtype2='s';
    else
        error('Unsupported datatype');
    end

    // Read binary data
    I = mtlb_fread(fid,prod(dimension(2:5)),dtype);

    if datatype==4 | datatype==132 then // This saves a lot of space..  
        I=int16(I);   
    elseif datatype==2 then
        I=unint8(I);
    elseif datatype==8 then
        I=int32(I);
    elseif datatype==16 then
        I=float(I);
    end



    // Close data file
    mclose(fid);

    if RESHAPE_DATA
        // Reshape data
        select dimension(1)
        case 4
            // 4-dimensional data
            if dimension(5) == 1
                I = matrix(I,dimension(2),dimension(3),dimension(4));
                if FLIP_DATA
                    I = flipdim(I,1);
                    I = flipdim(I,2);
                    I = flipdim(I,3);
                end
                // //////////////////////////////////////
            else
                I = matrix(I,dimension(2),dimension(3),dimension(4),dimension(5));
                if FLIP_DATA
                    I = flipdim(I,1);
                    I = flipdim(I,2);
                    I = flipdim(I,3);
                end
            end
        case 3
            // 3-dimensional data
            I = matrix(I,dimension(2),dimension(3),dimension(4));
            if FLIP_DATA
                I = flipdim(I,1);
                I = flipdim(I,2);
                I = flipdim(I,3);
            end
            // //////////////////////////////////////
        case 2
            // 2-dimensional data
            I = matrix(I,dimension(2),dimension(3));
        case 1
            // 1-d data
        else
            printf('Warning: data not reshaped\n');
        end
    end

    ii = find(pixdim);
    pixdim = pixdim(ii);


    save(filename+'.dat',I,pixdim,dtype,dimension);

endfunction




function [out,pixdim,rot,dtype] = read_nifti(filename,volnum)
// READNIFTI  - Read MRI files in the NIFTI format
//
//    usage: 
//            [data,pixdim,orient,dtype] = readnifti(filename);
//            [data,pixdim,orient,dtype] = readnifti(filename,volumenum);
//
//        filename - is the name of a file in NIFTI format. 
//                   (filename can include the .nii, .hdr, or .img
//                   extensions or just the base filename. If base
//                   filename is used then the file is assumed to have
//                   the .nii extension.)
//       volumenum - loads a specific volume from a 4D dataset
//
//            data - is a n-dimensional matrix of the data
//          pixdim - is the size of each voxel (mm)
//          orient - is a structure containing the qform and sform
//                   matrices.
//           dtype - is the datatype of the file (ie 'uint8', 'int16', etc)
//

// Written by Colin Humphries, 2005
// Modified and translated in Scilab by Davide Poggiali, PhD student Padua IT, June 2014
    // davide.poggiali@studenti.unipd.it
// //////// User parameters //////////////////////////////////////////
RESHAPE_DATA = 1; // When this flag is set to 1 the data is reshaped into
                  // an n-dimensional matrix. Otherwise readnifti will
                  // just output a vector. 
// ////////////////////////////////////////////////////////////////////////////////////

if argn(2) < 2
  volnum = [];
end

// Get rid of possible file extensions and figure out if we are dealing
// with a NIFTI .nii file or a NIFTI .img/.hdr file
ind = mtlb_findstr(filename,'.gz');
if ~isempty(ind)
  unix_w("gunzip "+filename)
  filename =strsubst(filename,'.gz','');
  //gzipped = 1;
  //error('Currently cannot open gzipped data files');
end
ind =mtlb_findstr(filename,'.nii');
if ~isempty(ind)
  filename =strsubst(filename,'.nii','');
  filenametype = 'nii';
else
  ind = mtlb_findstr(filename,'.img');
  if ~isempty(ind)
    filename =strsubst(filename,'.img','');
    filenametype = 'img';
  else
    ind = mtlb_findstr(filename,'.hdr');
    if ~isempty(ind)
      filename = filename(1:ind-1);
      filenametype = 'img';
    else
      filenametype = 'nii';
    end
  end
end

// Read header information
fileinfo = readniftiheader(filename,filenametype);

// Open Data File
if isempty(filenametype)
  error('not implemented yet');
else
  if mtlb_strcmp(filenametype,'img')
    fid = mopen(filename+'.img','r',fileinfo.byteswap);
    if fid < 0
      error('Error opening image file');
    end
  else
    fid = mopen(filename+'.nii','r',fileinfo.byteswap);
    if fid < 0
      error('Error opening image file');
    end
    mseek(fileinfo.voxoffset,fid,'set');
  end
end

// Convert to matlab datatypes
select fileinfo.datatype
  case 2
    dtype = 'uint8';
  case 4
    dtype = 'int16';
  case 8
    dtype = 'int32';
  case 16
    dtype = 'float';
  case 64
    dtype = 'double';
  case 132
    dtype = 'int16';
  else
    error('Unsupported datatype');
end
dimension = fileinfo.dimension;

// Read binary data
if isempty(volnum)
  out = mtlb_fread(fid,prod(dimension(2:4)),dtype);
else
  select fileinfo.datatype
   case 2
    mseek((volnum-1)*prod(dimension(2:4)),fid,'cur');
   case 4
    mseek(2*(volnum-1)*prod(dimension(2:4)),fid,'cur');
   case 8
    mseek(4*(volnum-1)*prod(dimension(2:4)),fid,'cur');
   case 16
    mseek(4*(volnum-1)*prod(dimension(2:4)),fid,'cur');
   case 64
    mseek(8*(volnum-1)*prod(dimension(2:4)),fid,'cur');
   case 132
    mseek(2*(volnum-1)*prod(dimension(2:4)),fid,'cur');
   else
    error('Unsupported datatype');
  end
  // dimension
  out = mtlb_fread(fid,prod(dimension(2:4)),dtype);
  RESHAPE_DATA = 0;
end
// Close data file
mclose(fid);

if RESHAPE_DATA
  // Reshape data
  select dimension(1)
    case 4
      // 4-dimensional data
      if dimension(5) == 1
	out = matrix(out,dimension(2),dimension(3),dimension(4));
	// //////////////////////////////////////
      else
	out = matrix(out,dimension(2),dimension(3),dimension(4), dimension(5));
      end
    case 3
      // 3-dimensional data

      out = matrix(out,dimension(2),dimension(3),dimension(4));
      // //////////////////////////////////////
    case 2
      // 2-dimensional data
      out = matrix(out,dimension(2),dimension(3));
    case 1
      // 1-d data
    else
      printf('Warning: data not reshaped\n');
  end
end
pixdim = fileinfo.pixdim(2:(dimension(1)+1));
rot = struct('qfac',fileinfo.pixdim(1),...
                'qform',fileinfo.formcodes(1),...
                'qmatrix',fileinfo.qmatrix,...
                'sform',fileinfo.formcodes(2),...
                'smatrix',fileinfo.smatrix);
I=out;
save(filename+'.dat',I,pixdim,dtype,dimension);
endfunction

function [fileinfo] = readniftiheader(filename,filenametype)
// Open Header
//    filename is the name of the file
//    filenametype is either 'img' or 'nii'
//

if isempty(filenametype)
  error('not implemented yet');
else
  if isequal(filenametype,'img')
    fid = mopen(filename+'.hdr','r');
    if fid < 0
      error('Error opening header file');
    end
  else
    fid = mopen(filename+'.nii','r');
    if fid < 0
      error('Error opening header file');
    end
  end
end

mseek(40,fid,'set');
dimension = mtlb_fread(fid,8,'int16');
byteswap = 1;

// The next part tries to figure out if we have a byte swapping problem by
// looking at the dimension bit in the header file. If it's bigger than 10
// (we probably aren't dealing with more than 10-dimensional data) then it
// tries byte swapping. If it still fails then it outputs an error.
// dimension
if (dimension(1) > 10)
  byteswap = 0;
  fclose(fid);
  if isempty(filenametype)
    error('not implemented yet');
  else
    if mtlb_strcmp(filenametype,'img')
      fid = fopen(filename+'.hdr','r',byteswap);
      if fid < 0
        error('Error opening header file');
      end
    else
      fid = mopen(filename+'.nii','r',byteswap);
      if fid < 0
        error('Error opening header file');
      end
    end
  end
  mseek(40,fid,'set');
  dimension = mtlb_fread(fid,8,'int16');
  if (dimension(1) > 10)
    byteswap = 1;
    mclose(fid);
    if isempty(filenametype)
      error('not implemented yet');
    else
      if mtlb_strcmp(filenametype,'img')
        fid = mopen(filename+'.hdr','r',byteswap);
        if fid < 0
          error('Error opening header file');
        end
      else
        fid = mopen(filename+'.nii','r',byteswap);
        if fid < 0
          error('Error opening header file');
        end
      end
    end
    mseek(40,fid,'set');
    dimension = mtlb_fread(fid,8,'int16');
    if (dimension(1) > 10)
      mclose(fid);
      error('Error opening file. Dimension argument is not valid');
    end
  end
end

// Read information from header file
// Note: there is information in the header that is not currently read
// like the intent codes.

// datatype
mseek(40+30,fid,'set');
datatype = mtlb_fread(fid,1,'int16');

// pixel dimension
mseek(40+36,fid,'set');
pixdim = mtlb_fread(fid,8,'float');

// bits per pixel
mseek(40+32,fid,'set');
bitpix = mtlb_fread(fid,1,'int16');

// data offset
mseek(108,fid,'set');
voxoffset = mget(1,'f',fid);

// orientation 
mseek(252,fid,'set');
formcodes = mtlb_fread(fid,2,'int16');
mseek(256,fid,'set');
qmatrix = mtlb_fread(fid,6,'float');
mseek(280,fid,'set');
smatrix = mtlb_fread(fid,12,'float');

// Close header file
mclose(fid);

fileinfo = struct('dimension',dimension,...
                  'pixdim',pixdim,...
                  'datatype',datatype,...
                  'bitpix',bitpix,...
                  'voxoffset',voxoffset,...
                  'byteswap',byteswap,...
                  'formcodes',formcodes,...
                  'qmatrix',qmatrix,...
                  'smatrix',smatrix);
  
endfunction
