function [] = writeanalyze(data,varargin)
    // WRITEANALYZE - Save MRI data in the Anaylze format
    //
    //     usage:
    //            writeanalyze(data,filename,vsize)
    //            writeanalyze(data,filename,vsize,dtype)
    //            writeanalyze(data,isize,filename,vsize)
    //            writeanalyze(data,isize,filename,vsize,index)
    //            writeanalyze(data,isize,filename,vsize,dtype)
    //            writeanalyze(data,isize,filename,vsize,dtype,index)
    //            writeanalyze('header',isize,filename,vsize,dtype)
    //
    //       data     - mri data
    //                  Data can be a matrix(3d,4d) or vector. If data is a 
    //                  vector then isize must be included. The data can also
    //                  be indexed, meaning that each point corresponds to an
    //                  indexed point in the 3d volume. In this case both the
    //                  index vector and isize must be included.
    //                  ( Note: 3d_data(index) = indexed_data 
    //                    for 4d data the same index is used for each time
    //                    point )
    //       isize    - size of data [X,Y,Z] or [X,Y,Z,T]
    //       filename - name of output file (with or without extension)
    //       vsize    - size of voxels [X,Y,Z] (mm)
    //       dtype    - data type ('uint8','int16','double',etc...)
    //                  The default data type is the same as the input data
    //                  matrix. Use this parameter to set the data type to be
    //                  something different.
    //       index    - index vector.
    //
    //       If the 'header' option is used writeanalyze will only write a
    //       header file. This is useful if the data file already exists and
    //       a header file is needed.
    //

    // Written by Colin Humphries
    // Feb, 2000
    // University of California, Irvine
    // colin@alumni.caltech.edu

    // Updated Sep 2003 - Changed parameter list
    //                  - Can now handle 4-d data
    //         Feb 2004 - Added SPM parameters

    // Copywrite (c) 2003, Colin J. Humphries, All Rights Reserved

    // Info on Analyze format: 
    //    http://www.mayo.edu/bir/PDF/ANALYZE75.pdf
    //


    // Modified and translated in Scilab by Davide Poggiali, PhD student Padua IT, June 2014
    // davide.poggiali@studenti.unipd.it

DEFAULT_SCALING =[]

   //DEFAULT_SCALING = 'pos';  // Note: this variable is included to work with
    // AIR which uses the global scaling variables
    // which can be set in the header. Most programs
    // don't use these variables.
    // Values: 
    // 'pos' 0 and largest possible value. ie for
    //    uint8 [0 255] or for 'int16' [0 32767]
    // 'posneg' smallest and largest possible values
    //    ie for int16 [-32767 32767]
    // 'auto' checks if there are any negative
    //    values in which case 'posneg' is used
    //    otherwise 'pos' is used.
    // [] use the min and max of the data
    // If you don't use this variable then you might
    // as well use [0,0] which will set the bits to
    // zeros.

    if ~(type(data)==10)
        isize = [];
        dtype = [];
        index = [];
    end
    // Get arguments from list
    select argn(2)
    case 6
        isize = varargin(1);
        filename = varargin(2);
        vsize = varargin(3);
        dtype = varargin(4);
        index = varargin(5);
    case 5
        isize = varargin(1);
        filename = varargin(2);
        vsize = varargin(3);
        if isstr(varargin(4))
            dtype = varargin(4);
        else
            index = varargin(4);
        end
    case 4
        if type(varargin(1))==10
            filename = varargin(1);
            vsize = varargin(2);
            dtype = varargin(3);
        else
            isize = varargin(1);
            filename = varargin(2);
            vsize = varargin(3);
        end
    case 3
        filename = varargin(1);
        vsize = varargin(2);
            case 2
        filename = varargin(1);
        vsize = [1 1 1];
    else
        error('Wrong number of inputs.');
    end

    // Get isize and dtype from the data
    if isempty(isize)
        isize = size(data);
    end  
    if isempty(dtype)
        dtype = typeof(data(1));
    end
    if dtype=='constant' then
        dtype='double';
    end
    // Get rid of possible file extensions in filename
    ind = mtlb_findstr(filename,'.img');
    if ~isempty(ind)
        filename = filename(1:ind-1);
    end
    ind = mtlb_findstr(filename,'.hdr');
    if ~isempty(ind)
        filename = filename(1:ind-1);
    end

    // If the type is logical then use uint8
    if dtype=='logical'
        dtype = 'uint8';
    end
    // convert dtype in scilab format
    select dtype
    case 'uint8'
        dtype2='us';
    case 'int16'
        dtype2='s';
    case 'int32'
        dtype2='i';
    case 'float'
        dtype2='f';
    case 'double'
        dtype2='d';
    case 'int16'
        dtype2='s';
    else
        error('Unsupported datatype');
    end

    if length(isize) == 3
        // Output 3-d data
        if ~isempty(index)
            ndata = zeros(1,prod(isize));
            ndata(index) = data;
        else
            ndata = data;
        end
        if isempty(DEFAULT_SCALING)
            DEFAULT_SCALING = [min(ndata(:)) max(ndata(:))];
        end
        if type(DEFAULT_SCALING)==10
            if strcmp(DEFAULT_SCALING,'auto')
                if or(ndata(:) < 0)
                    DEFAULT_SCALING = 'posneg';
                else
                    DEFAULT_SCALING = 'pos';
                end
            end
        end

        // Write header file
        writeheaderfile(filename,isize,vsize,dtype,DEFAULT_SCALING);

        // Open img file
        fid = mopen(filename+'.img','w');
        if fid < 0
            error('Can''t open .img file.');
        end
        // Output data

        mput(ndata(:),dtype2,fid);
        mclose(fid);
    elseif length(isize) == 4
        // Output 4-d data 
        // The 4d data is treated separately here because the indexing is
        // different.
        if isempty(DEFAULT_SCALING)
            DEFAULT_SCALING = [min(data(:)) max(data(:))];
        end
        if type(DEFAULT_SCALING)==10
            if strcmp(DEFAULT_SCALING,'auto')
                if or(ndata(:) < 0)
                    DEFAULT_SCALING = 'posneg';
                else
                    DEFAULT_SCALING = 'pos';
                end
            end
        end

        // Write header file
        writeheaderfile(filename,isize,vsize,dtype,DEFAULT_SCALING);

        // Open img file
        fid = mopen(filename+'.img','w');
        if fid < 0
            error('Can''t open .img file.');
        end
        if ~isempty(index)
            for ii = 1:isize(4)
                ndata = zeros(1,prod(isize(1:3)));
                ndata(index) = data((ii-1)*length(index)+1:ii*length(index));
                mtlb_fwrite(fid,ndata,dtype);
            end
        else
            mtlb_fwrite(fid,data,dtype);
        end

        mclose(fid);
    end
endfunction

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
function [] = writeheaderfile(filename,isize,vsize,dtype,defscaling)
    // Internal function for creating a header file.
    //

    // Switch from matlab datatype to the AIR datatype
    select dtype
    case 'uint8'
        datatype = 2;  
        bitpix = 8;
    case 'int16'
        datatype = 4;
        bitpix = 16; 
    case 'short'
        datatype = 4;
        bitpix = 16; 
    case 'int32'
        datatype = 8;
        bitpix = 32;
    case 'int'
        datatype = 8;
        bitpix = 32;
    case 'single'
        datatype = 16;
        bitpix = 32;
    case 'float32'
        datatype = 16;
        bitpix = 32;
    case 'float'
        datatype = 16;
        bitpix = 32;
    case 'double'
        datatype = 64;
        bitpix = 64;
    case 'float64'
        datatype = 64;
        bitpix = 64;
    else
        error('Unsupported datatype.');
    end

    // Set the scaling values.
    if type(defscaling)==10
        select defscaling
        case 'default'
            if bitpix == 8
                defscaling = [0 255];
            elseif bitpix == 16
                defscaling = [0 32767];
            else
                // Note: currently only 8 and 16 bit integers are set correctly.
                // because AIR only uses those datatypes.
                defscaling = [0 32767];
            end
        case 'pos'
            if bitpix == 8
                defscaling = [0 255];
            elseif bitpix == 16
                defscaling = [0 32767];
            else
                // Note: currently only 8 and 16 bit integers are set correctly.
                // because AIR only uses those datatypes.
                defscaling = [0 32767];
            end
        case 'posneg'
            if bitpix == 8
                defscaling = [0 255];
            elseif bitpix  == 16
                defscaling = [-32767 32767];
            else
                // Note: currently only 8 and 16 bit integers are set correctly.
                defscaling = [-32767 32767];
            end
        end
    end
    // Open header file	      
    fid = mopen(filename+'.hdr','w');
    if fid < 0
        error('Can''t open header file');
    end

    mput(zeros(1,348),'us',fid);

    mseek(0,fid);
    mtlb_fwrite(fid,348,'int16');

    mseek(32,fid);
    mtlb_fwrite(fid,16384,'int16');

    mseek(38,fid);
    mputl('r',fid);

    mseek(40+0,fid);
    // This is the convention used for isize that I've seen in other Analyze
    // files. You could also use this convention (eg for 3d data
    // fwrite(fid,[3,isize(1),isize(2),isize(3)],'int16'); ) and it should
    // still be valid.
    select length(isize)
    case 1
        mtlb_fwrite(fid,[4,isize(1),1,1,1],'int16');
    case 2
        mtlb_fwrite(fid,[4,isize(1),isize(2),1,1],'int16');
    case 3
        mtlb_fwrite(fid,[4,isize(1),isize(2),isize(3),1],'int16');
    case 4
        mtlb_fwrite(fid,[4,isize(1),isize(2),isize(3),isize(4)],'int16');
    end

    mseek(40+30,fid);
    mtlb_fwrite(fid,datatype,'int16');

    mseek(40+32,fid);
    mtlb_fwrite(fid,bitpix,'int16');

    mseek(40+36,fid);
    mtlb_fwrite(fid,[0,vsize(:)'],'float32');

    mseek(40+100,fid);
    mtlb_fwrite(fid,[defscaling(2) defscaling(1)],'int32');

    // Note the following two fields are used by SPM and are not part of the
    // Analyze format.
    // Scale factor = 1
    mseek(40+72,fid);
    mtlb_fwrite(fid,1,'int32');
    // DC offset = 0
    mseek(40+76,fid);
    mtlb_fwrite(fid,0,'int32');

    mclose(fid);


endfunction
